@@ -6,6 +6,7 @@ Usefull objects to read and write the binary fields produced by XCompact3d.
 from __future__ import annotations
 
 import glob
+import io
 import os
 import os.path
 import warnings
@@ -235,6 +236,8 @@ class Dataset(traitlets.HasTraits):
     ----------
     data_path : str
         The path to the folder where the binary fields are located (default is ``"./data/"``).
+        .. note :: the default ``"./data/"`` is relative to the path to the parameters
+           file when initialized from :obj:`xcompact3d_toolbox.parameters.ParametersExtras`.
     drop_coords : str
         If working with two-dimensional planes, specify which of the coordinates should be
         dropped, i.e., ``"x"``, ``"y"`` or ``"z"``, or leave it empty for 3D fields (default is ``""``).
@@ -248,7 +251,7 @@ class Dataset(traitlets.HasTraits):
         need to speedup your application
         (default is an empty set).
     snapshot_counting : str
-        The parameter that controls the number of timesteps used to produce tha datasets
+        The parameter that controls the number of timesteps used to produce the datasets
         (default is ``"ilast"``).
     snapshot_step : str
         The parameter that controls the number of timesteps between each snapshot, it is often
@@ -262,7 +265,7 @@ class Dataset(traitlets.HasTraits):
 
     Notes
     -----
-        * :obj:`Dataset` is in fact an atribute of :obj:`xcompact3d_toolbox.parameters. ParametersExtras`,
+        * :obj:`Dataset` is in fact an atribute of :obj:`xcompact3d_toolbox.parameters.ParametersExtras`,
           so there is no need to initialize it manually for most of the common use cases.
 
         * All arrays are wrapped into Xarray objects (:obj:`xarray.DataArray`
@@ -300,6 +303,9 @@ class Dataset(traitlets.HasTraits):
 
         Parameters
         ----------
+        filename_properties : dict, optional
+            Keyword arguments for :obj:`FilenamePropertie`, like
+            :obj:`separator`, :obj:`file_extension` and so on.
         **kwargs
             Keyword arguments for the parameters, like :obj:`data_path`, :obj:`drop_coords` and so on.
 
@@ -343,10 +349,14 @@ class Dataset(traitlets.HasTraits):
         Initial setup:
 
         >>> prm = xcompact3d_toolbox.Parameters(loadfile="input.i3d")
-        >>> prm.dataset.filename_properties.set(
-        ...     separator = "-",
-        ...     file_extension = ".bin",
-        ...     number_of_digits = 3
+        >>> prm.dataset.set(
+        ...     filename_properties=dict(
+        ...         separator="-",
+        ...         file_extension=".bin",
+        ...         number_of_digits=3
+        ...     ),
+        ...     stack_scalar=True,
+        ...     stack_velocity=True,
         ... )
 
         Iterate over some snapshots, loading them one by one, with the same
@@ -396,10 +406,15 @@ class Dataset(traitlets.HasTraits):
         Initial setup:
 
         >>> prm = xcompact3d_toolbox.Parameters(loadfile="input.i3d")
-        >>> prm.dataset.filename_properties.set(
-        ...     separator = "-",
-        ...     file_extension = ".bin",
-        ...     number_of_digits = 3
+        >>> prm.dataset.set(
+        ...     filename_properties=dict(
+        ...         separator="-",
+        ...         file_extension=".bin",
+        ...         number_of_digits=3
+        ...     ),
+        ...     drop_coords="z",
+        ...     stack_scalar=True,
+        ...     stack_velocity=True,
         ... )
 
         * Load the entire time series for a given variable:
@@ -433,7 +448,7 @@ class Dataset(traitlets.HasTraits):
         elif isinstance(arg, slice):
             start, stop, step = arg.indices(len(self))
             return xr.concat(
-                [self.load_snapshot(t) for t in range(start, stop, step)], "t"
+                (self.load_snapshot(t) for t in range(start, stop, step)), "t"
             )
         elif isinstance(arg, str):
             return self.load_time_series(arg)
@@ -471,10 +486,14 @@ class Dataset(traitlets.HasTraits):
         Initial setup:
 
         >>> prm = xcompact3d_toolbox.Parameters(loadfile="input.i3d")
-        >>> prm.dataset.filename_properties.set(
-        ...     separator = "-",
-        ...     file_extension = ".bin",
-        ...     number_of_digits = 3
+        >>> prm.dataset.set(
+        ...     filename_properties=dict(
+        ...         separator="-",
+        ...         file_extension=".bin",
+        ...         number_of_digits=3
+        ...     ),
+        ...     stack_scalar=True,
+        ...     stack_velocity=True,
         ... )
 
         Iterate over all snapshots, loading them one by one:
@@ -508,11 +527,14 @@ class Dataset(traitlets.HasTraits):
 
         Parameters
         ----------
+        filename_properties : dict, optional
+            Keyword arguments for :obj:`FilenameProperties`, like
+            :obj:`separator`, :obj:`file_extension` and so on.
         **kwargs
             Keyword arguments for the parameters, like :obj:`data_path`, :obj:`drop_coords` and so on.
 
         Raises
-        -------
+        ------
         KeyError
             Exception is raised when an Keyword arguments is not a valid parameter.
 
@@ -520,14 +542,20 @@ class Dataset(traitlets.HasTraits):
         --------
 
         >>> prm = xcompact3d_toolbox.Parameters(loadfile="input.i3d")
-        >>> prm.dataset.filename_properties.set(
-        ...     separator = "-",
-        ...     file_extension = ".bin",
-        ...     number_of_digits = 3,
-        ...     data_path = "./data/"
+        >>> prm.dataset.set(
+        ...     filename_properties=dict(
+        ...         separator="-",
+        ...         file_extension=".bin",
+        ...         number_of_digits=3
+        ...     ),
+        ...     stack_scalar=True,
+        ...     stack_velocity=True,
         ... )
         """
 
+        if "filename_properties" in kwargs:
+            self.filename_properties.set(**kwargs.pop("filename_properties"))
+
         for key, arg in kwargs.items():
             if key not in self.trait_names():
                 raise KeyError(f"{key} is not a valid argument for Dataset")
@@ -560,24 +588,29 @@ class Dataset(traitlets.HasTraits):
         Initial setup:
 
         >>> prm = xcompact3d_toolbox.Parameters(loadfile="input.i3d")
-        >>> prm.dataset.filename_properties.set(
-        ...     separator = "-",
-        ...     file_extension = ".bin",
-        ...     number_of_digits = 3
+        >>> prm.dataset.set(
+        ...     filename_properties=dict(
+        ...         separator="-",
+        ...         file_extension=".bin",
+        ...         number_of_digits=3
+        ...     ),
+        ...     stack_scalar=True,
+        ...     stack_velocity=True,
         ... )
 
         Load one array from the disc:
 
-        >>> ux = prm.dataset.load_array("ux-0000.bin")
+        >>> ux = prm.dataset.load_array("ux-000.bin")
 
         """
 
         coords = self._mesh.drop(*self.drop_coords)
 
-        time_int, name = self.filename_properties.get_info_from_filename(filename)
-
         if add_time:
+            time_int, name = self.filename_properties.get_info_from_filename(filename)
             coords["t"] = [param["mytype"](self._time_step * time_int)]
+        else:
+            name = None
 
         # We obtain the shape for np.fromfile from the coordinates
         shape = [len(value) for value in coords.values()]
@@ -639,10 +672,14 @@ class Dataset(traitlets.HasTraits):
         Initial setup:
 
         >>> prm = xcompact3d_toolbox.Parameters(loadfile="input.i3d")
-        >>> prm.dataset.filename_properties.set(
-        ...     separator = "-",
-        ...     file_extension = ".bin",
-        ...     number_of_digits = 3
+        >>> prm.dataset.set(
+        ...     filename_properties=dict(
+        ...         separator="-",
+        ...         file_extension=".bin",
+        ...         number_of_digits=3
+        ...     ),
+        ...     stack_scalar=True,
+        ...     stack_velocity=True,
         ... )
 
         Load all variables from a given snapshot:
@@ -702,7 +739,14 @@ class Dataset(traitlets.HasTraits):
             return name in {"ux", "uy", "uz"}
 
         if stack_scalar:
-            scalar_variables = sorted(list(filter(is_scalar, set_of_variables,)))
+            scalar_variables = sorted(
+                list(
+                    filter(
+                        is_scalar,
+                        set_of_variables,
+                    )
+                )
+            )
 
             if scalar_variables:
                 dataset["phi"] = (
@@ -758,10 +802,14 @@ class Dataset(traitlets.HasTraits):
         Initial setup:
 
         >>> prm = xcompact3d_toolbox.Parameters(loadfile="input.i3d")
-        >>> prm.dataset.filename_properties.set(
-        ...     separator = "-",
-        ...     file_extension = ".bin",
-        ...     number_of_digits = 3
+        >>> prm.dataset.set(
+        ...     filename_properties=dict(
+        ...         separator="-",
+        ...         file_extension=".bin",
+        ...         number_of_digits=3
+        ...     ),
+        ...     stack_scalar=True,
+        ...     stack_velocity=True,
         ... )
 
         Load the entire time series for a given variable:
@@ -793,22 +841,26 @@ class Dataset(traitlets.HasTraits):
             raise IOError(f"No file was found corresponding to {filename_pattern}.")
 
         return xr.concat(
-            [
+            (
                 self.load_array(file, add_time=True)
                 for file in tqdm(filename_list, desc=filename_pattern)
-            ],
+            ),
             dim="t",
         )
 
-    def load_wind_turbine_data(self, file_pattern: str) -> Type[xr.Dataset]:
+    def load_wind_turbine_data(self, file_pattern: str = None) -> Type[xr.Dataset]:
         """Load the data produced by wind turbine simulations.
 
         .. note:: This feature is experimental
 
         Parameters
         ----------
-        file_pattern : str
-            A filename pattern used to locate the files with :obj:`glob.glob`.
+        file_pattern : str, optional
+            A filename pattern used to locate the files with :obj:`glob.iglob`.
+            If None, it is obtained from ``datapath``, i.g.,
+            if ``datapath = ./examples/Wind-Turbine/data`` then
+            ``file_pattern = ./examples/Wind-Turbine/data/../*.perf``.
+            By default None.
 
         Returns
         -------
@@ -819,7 +871,7 @@ class Dataset(traitlets.HasTraits):
         --------
 
         >>> prm = xcompact3d_toolbox.Parameters(loadfile="NREL-5MW.i3d")
-        >>> ds = prm.dataset.load_wind_turbine_data("*.perf")
+        >>> ds = prm.dataset.load_wind_turbine_data()
         >>> ds
         <xarray.Dataset>
         Dimensions:          (t: 21)
@@ -840,13 +892,15 @@ class Dataset(traitlets.HasTraits):
             Torque           (t) float64 8.78e+06 1.268e+06 ... 4.231e+06 4.203e+06
             Power            (t) float64 1.112e+07 1.952e+06 ... 5.362e+06 5.328e+06
         """
+
         def get_dataset(filename):
 
             time = os.path.basename(filename).split("_")[0]
 
-            time = int(time) * self._prm.iturboutput * self._prm.dt
+            time = float(time) * self._prm.iturboutput * self._prm.dt
 
             ds = xr.Dataset(coords=dict(t=[time]))
+            ds.t.attrs = dict(name="t", long_name="Time", units="s")
 
             for name, values in pd.read_csv(filename).to_dict().items():
                 ds[name.strip()] = xr.DataArray(
@@ -857,9 +911,12 @@ class Dataset(traitlets.HasTraits):
 
             return ds
 
-        filenames = glob.glob(file_pattern)
+        if file_pattern is None:
+            file_pattern = os.path.join(self.data_path, "..", "*.perf")
 
-        return xr.concat([get_dataset(file) for file in filenames], dim="t").sortby("t")
+        filenames = glob.iglob(file_pattern)
+
+        return xr.concat((get_dataset(file) for file in filenames), dim="t").sortby("t")
 
     def write(self, data: Union[xr.DataArray, xr.Dataset], file_prefix: str = None):
         """Write an array or dataset to raw binary files on the disc, in the
@@ -898,10 +955,14 @@ class Dataset(traitlets.HasTraits):
         Initial setup:
 
         >>> prm = xcompact3d_toolbox.Parameters(loadfile="input.i3d")
-        >>> prm.dataset.filename_properties.set(
-        ...     separator = "-",
-        ...     file_extension = ".bin",
-        ...     number_of_digits = 3
+        >>> prm.dataset.set(
+        ...     filename_properties=dict(
+        ...         separator="-",
+        ...         file_extension=".bin",
+        ...         number_of_digits=3
+        ...     ),
+        ...     stack_scalar=True,
+        ...     stack_velocity=True,
         ... )
 
         * From a dataset, write only the variables with the atribute ``file_name``,
@@ -926,10 +987,17 @@ class Dataset(traitlets.HasTraits):
           ...     vort = ds.uy.x3d.first_derivative("x") - ds.ux.x3d.first_derivative("y")
           ...     prm.dataset.write(data = vort, file_prefix = "vorticity")
 
+        .. note :: It is not recomended to load the arrays with
+           ``add_time = False`` when planning to write the results in a
+           time series (e.g., `vort-000.bin`, `vort-001.bin`, `vort-002.bin`, ...)
+
+
         """
         if isinstance(data, xr.Dataset):
+            os.makedirs(self.data_path, exist_ok=True)
             self._write_dataset(data)
         elif isinstance(data, xr.DataArray):
+            os.makedirs(self.data_path, exist_ok=True)
             self._write_array(data, file_prefix)
         else:
             raise IOError(
@@ -976,7 +1044,8 @@ class Dataset(traitlets.HasTraits):
                 self._write_array(
                     dataArray.isel(t=k, drop=True),
                     self.filename_properties.get_filename_for_binary(
-                        prefix=filename, counter=int(time / dt),
+                        prefix=filename,
+                        counter=int(time / dt),
                     ),
                 )
         # and finally writes to the disc
@@ -1016,10 +1085,14 @@ class Dataset(traitlets.HasTraits):
         Initial setup:
 
         >>> prm = xcompact3d_toolbox.Parameters(loadfile="input.i3d")
-        >>> prm.dataset.filename_properties.set(
-        ...     separator = "-",
-        ...     file_extension = ".bin",
-        ...     number_of_digits = 3
+        >>> prm.dataset.set(
+        ...     filename_properties=dict(
+        ...         separator="-",
+        ...         file_extension=".bin",
+        ...         number_of_digits=3
+        ...     ),
+        ...     stack_scalar=True,
+        ...     stack_velocity=True,
         ... )
 
         It is possible to produce a new xdmf file, so all data
@@ -1040,7 +1113,10 @@ class Dataset(traitlets.HasTraits):
                 raise IOError(f"No file was found corresponding to {filename_pattern}.")
 
             properties = zip(
-                *map(self.filename_properties.get_info_from_filename, filename_list,)
+                *map(
+                    self.filename_properties.get_info_from_filename,
+                    filename_list,
+                )
             )
             time_numbers, var_names = properties
             time_numbers = sorted(list(set(time_numbers)))
@@ -1206,13 +1282,20 @@ def prm_to_dict(filename="incompact3d.prm"):
     return dict_outer
 
 
-def i3d_to_dict(filename="input.i3d"):
+def i3d_to_dict(filename="input.i3d", string=None):
 
-    f = open(filename)
+    if string is None:
+        with open(filename, "r") as f:
+            string = f.read()
+
+    return i3d_string_to_dict(string)
+
+
+def i3d_string_to_dict(string):
 
     dict_outer = {}
 
-    for line in f:
+    for line in io.StringIO(string):
         # Remove comments
         line = line.partition("!")[0].replace(" ", "")
         # Remove spaces
@@ -1260,6 +1343,4 @@ def i3d_to_dict(filename="input.i3d"):
         else:  # Not a list
             dict_inner[param] = value
 
-    f.close()
-
     return dict_outer
