@@ -15,7 +15,9 @@ For more details, see:
 
 import os.path
 
+import numba
 import numpy as np
+import stl
 import xarray as xr
 
 from .array import X3dDataArray, X3dDataset
@@ -89,7 +91,7 @@ def init_epsi(prm, dask=False):
     # for integer ao float as well
     for key, (x, y, z) in fields.items():
         epsi[key] = xr.DataArray(
-            np.zeros((x.size, y.size, z.size), dtype=np.bool),
+            np.zeros((x.size, y.size, z.size), dtype=bool),
             dims=["x", "y", "z"],
             coords={"x": x, "y": y, "z": z},
             name=key,
@@ -280,6 +282,171 @@ class Geometry:
     def __init__(self, data_array):
         self._data_array = data_array
 
+    def from_stl(
+        self,
+        filename: str = None,
+        stl_mesh: stl.mesh.Mesh = None,
+        origin: dict = None,
+        rotate: dict = None,
+        scale: float = None,
+        user_tol: float = 2.0 * np.pi,
+        remp: bool = True,
+    ):
+        """Load a STL file and compute if the nodes of the computational
+        mesh are inside or outside the object. In this way, the
+        customized geometry can be used at the flow solver.
+
+        The methodology is based on the work of:
+
+        * Jacobson, A., Kavan, L., & Sorkine-Hornung,
+          O. (2013). Robust inside-outside segmentation
+          using generalized winding numbers. ACM
+          Transactions on Graphics (TOG), 32(4), 1-12.
+
+        The Python implementation is an adaptation from
+        `inside-3d-mesh <https://github.com/marmakoide/inside-3d-mesh>`_
+        (licensed under the MIT License),
+        by `@marmakoide <https://github.com/marmakoide>`_.
+
+        To maximize the performance here at the toolbox, :obj:`from_stl` is powered by
+        `Numba`_, that translates Python functions to optimized machine code at runtime.
+        This method is compatible with `Dask`_ for parallel computation.
+        In addition, just the subdomain near the object is tested, to save computational
+        time.
+
+        .. note:: The precision of the method is influenced by the
+           complexity of the STL mesh, there is no guarantee it will work
+           for all geometries. This feature is experimental, its
+           interface may change in future releases.
+
+
+        Parameters
+        ----------
+        filename : str, optional
+            Filename of the STL file to be loaded and included in the cartesian domain, by default None
+        scale : float, optional
+            This parameters can be used to scale up the object when greater than one and scale it down when smaller than one, by default None
+        rotate : dict, optional
+            Rotate the object, including keyword arguments that are
+            expected by :obj:`stl.mesh.Mesh.rotate`, like ``axis``,
+            ``theta`` and ``point``.
+            For more details, see `numpy-stl's documentation`_.
+            By default None
+        origin : dict, optional
+            Specify the location of the origin point for the geometry.
+            It is considered as the minimum value computed from all
+            points in the object for each coordinate, after scaling
+            and rotating them. The keys of the dictionary are the
+            coordinate names (``x``, ``y`` and ``z``) and the values are the
+            origin on that coordinate.
+            For missing keys, the value is assumed as zero.
+            By default None
+        stl_mesh : stl.mesh.Mesh, optional
+            For very customizable control over the 3D object, you
+            can provide it directly. Note that none of the arguments
+            above are applied in this case.
+            For more details about how to create and modify the
+            geometry, see `numpy-stl's documentation`_.
+            By default None
+        user_tol : float, optional
+            Control the tolerance used to compute if a mesh node is
+            inside or outside the object. Values smaller than the default may reduce the number of false negatives.
+            By default :math:`2\pi`
+        remp : bool, optional
+            Add the geometry to the :obj:`xarray.DataArray` if
+            :obj:`True` and removes it if :obj:`False`, by default True
+
+        Returns
+        -------
+        :obj:`xarray.DataArray`
+            Array with(out) the customized geometry
+
+        Raises
+        -------
+        ValueError
+            If neither :obj:`filename` or :obj:`stl_mesh` are specified
+        ValueError
+            If :obj:`stl_mesh` is not valid, the test is performed by :obj:`stl.mesh.Mesh.check`
+        ValueError
+            If :obj:`stl_mesh` is not closed, the test is performed by
+            :obj:`stl.mesh.Mesh.is_closed`
+
+
+        Examples
+        --------
+
+        >>> prm = xcompact3d_toolbox.Parameters()
+        >>> epsi = xcompact3d_toolbox.init_epsi(prm, dask = True)
+        >>> for key in epsi.keys():
+        >>>     epsi[key] = epsi[key].geo.from_stl(
+        ...         "My_file.stl",
+        ...         scale=1.0,
+        ...         rotate=dict(axis=[0, 0.5, 0], theta=math.radians(90)),
+        ...         origin=dict(x=2, y=1, z=0),
+        ...     )
+
+        .. _`Dask`: https://dask.org/
+        .. _`numpy-stl's documentation`: https://numpy-stl.readthedocs.io/en/latest/
+        .. _`Numba`: http://numba.pydata.org/
+
+        """
+
+        def get_boundary(mesh_coord, coord):
+            """Get the boundaries of the subdomain for a given direction (x, y, z)
+            near the object. It returns a tuple of integers, representing the min and max
+            indexes of the coordinate where we need to loop through.
+            """
+            min = coord.searchsorted(mesh_coord.min(), "left")
+            max = coord.searchsorted(mesh_coord.max(), "right")
+            return min, max
+
+        if filename is not None and stl_mesh is None:
+            stl_mesh = stl.mesh.Mesh.from_file(filename)
+
+            if scale is not None:
+                stl_mesh.vectors *= scale
+
+            if rotate is not None:
+                stl_mesh.rotate(**rotate)
+
+            if origin is None:
+                origin = {}
+
+            stl_mesh.translate(
+                [
+                    origin.get("x", 0.0) - stl_mesh.x.min(),
+                    origin.get("y", 0.0) - stl_mesh.y.min(),
+                    origin.get("z", 0.0) - stl_mesh.z.min(),
+                ]
+            )
+
+        if stl_mesh is None:
+            raise ValueError("Please, specify filename or stl_mesh")
+
+        if not stl_mesh.check():
+            raise ValueError("stl_mesh is not valid")
+
+        if not stl_mesh.is_closed():
+            raise ValueError("stl_mesh is not closed")
+
+        x = self._data_array.x.data
+        y = self._data_array.y.data
+        z = self._data_array.z.data
+
+        return self._data_array.where(
+            ~_geometry_inside_mesh(
+                stl_mesh.vectors.astype(np.longdouble),
+                x.astype(np.longdouble),
+                y.astype(np.longdouble),
+                z.astype(np.longdouble),
+                user_tol,
+                get_boundary(stl_mesh.x, x),
+                get_boundary(stl_mesh.y, y),
+                get_boundary(stl_mesh.z, z),
+            ),
+            remp,
+        )
+
     def cylinder(self, radius=0.5, axis="z", height=None, remp=True, **kwargs):
         """Draw a cylinder.
 
@@ -703,3 +870,72 @@ class Geometry:
             self._data_array[dim] <= self._data_array[dim][-1] / 2.0,
             self._data_array[{dim: slice(None, None, -1)}].values,
         )
+
+
+@numba.njit
+def _geometry_inside_mesh(triangles, x, y, z, user_tol, lim_x, lim_y, lim_z):
+
+    result = np.zeros((x.size, y.size, z.size), dtype=numba.boolean)
+
+    for i in range(*lim_x):
+        for j in range(*lim_y):
+            for k in range(*lim_z):
+                result[i, j, k] = _point_in_geometry(
+                    triangles, x[i], y[j], z[k], user_tol
+                )
+
+    return result
+
+
+@numba.njit
+def _anorm2(X):
+    # Compute euclidean norm
+    return np.sqrt(np.sum(X ** 2.0))
+
+
+@numba.njit
+def _adet(X, Y, Z):
+    # Compute 3x3 determinant
+    ret = np.multiply(np.multiply(X[0], Y[1]), Z[2])
+    ret += np.multiply(np.multiply(Y[0], Z[1]), X[2])
+    ret += np.multiply(np.multiply(Z[0], X[1]), Y[2])
+    ret -= np.multiply(np.multiply(Z[0], Y[1]), X[2])
+    ret -= np.multiply(np.multiply(Y[0], X[1]), Z[2])
+    ret -= np.multiply(np.multiply(X[0], Z[1]), Y[2])
+    return ret
+
+
+@numba.njit
+def _point_in_geometry(triangles, x, y, z, user_tol):
+    """
+    The methodology is based on the work of:
+
+    * Jacobson, A., Kavan, L., & Sorkine-Hornung,
+        O. (2013). Robust inside-outside segmentation
+        using generalized winding numbers. ACM
+        Transactions on Graphics (TOG), 32(4), 1-12.
+
+    The Python implementation is an adaptation of
+    `inside-3d-mesh <https://github.com/marmakoide/inside-3d-mesh>`_,
+    by `Devert Alexandre <https://github.com/marmakoide>`_,
+    licensed under the MIT License.
+    """
+    X = np.array((x, y, z), dtype=triangles.dtype)
+
+    # One generalized winding number per input vertex
+    ret = triangles.dtype.type(0.0)
+
+    # Accumulate generalized winding number for each triangle
+    for U, V, W in triangles:
+        A, B, C = U - X, V - X, W - X
+        omega = _adet(A, B, C)
+
+        a, b, c = _anorm2(A), _anorm2(B), _anorm2(C)
+        d = a * b * c
+        d += c * np.sum(np.multiply(A, B))
+        d += a * np.sum(np.multiply(B, C))
+        d += b * np.sum(np.multiply(C, A))
+
+        ret += np.arctan2(omega, d)
+
+    return ret >= user_tol
